//
//  YHAIViewController.swift
//  galaxy
//
//  Created by Dufet on 2024/11/23.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit
import Alamofire
import IQKeyboardManagerSwift
import JXSegmentedView

class YHAIMainChatViewController: YHBaseViewController {

    var robotId: String = ""
    var conversationId: String = ""
    var isNeedAutoResponseImage = false
    var messages:[YHAIChatMessage] = []
    
    let manager = YHAIRequestManager()
    let viewModel = YHAIViewModel()
    
    lazy var chatConfig: YHAIChatConfiguration = {
        let config = YHAIChatConfiguration()
        return config
    }()

    lazy var tableView: UITableView = {
        
        let tableView = UITableView(frame:.zero, style:.grouped)

        if #available(iOS 11.0, *) {
            tableView.contentInsetAdjustmentBehavior = .never
        }
        tableView.showsVerticalScrollIndicator = false
        tableView.estimatedSectionHeaderHeight = 1.0
        tableView.estimatedSectionFooterHeight = 1.0
        tableView.backgroundColor = .clear
        tableView.separatorStyle = .none
        tableView.delegate = self
        tableView.dataSource = self
        chatConfig.registerMessageGroupCells(tableView)
        return tableView
    }()
    
    lazy var bottomInputView: YHAITextInputView = {
        let v = YHAITextInputView(frame: .zero)
        v.sendBlock = {
            [weak self] text in
            guard let self = self else { return }
            self.sendMessage(text)
        }
        v.stopSendBlock = {
            [weak self] in
            guard let self = self else { return }
            self.manager.stopChat(chatId: self.manager.chatId, conversationId: self.conversationId) { success, error in
                self.bottomInputView.status = self.bottomInputView.textView.text.isEmpty ? .disableSend : .enableSend
                if success {
                    self.chatConfig.disableHandleMessage = true
                    self.chatConfig.removeThinkingMessageFromChatList(&self.messages)
                    self.bottomInputView.status = self.bottomInputView.textView.text.isEmpty ? .disableSend : .enableSend
                    self.tableView.reloadData()
                }
            }
        }
        v.keyBoardChangeBlock = {
            [weak self] isKeyBoardShow in
            guard let self = self else { return }
            self.scrollToBottom()
        }
        return v
    }()
    
    lazy var thinkingMessage: YHAIChatMessage = {
        let msg = YHAIChatMessage.createThinkingMesssage()
        return msg
    }()
    
    override func viewDidLoad() {
        
        super.viewDidLoad()
        gk_navBarAlpha = 0
        gk_navigationBar.isHidden = true
        view.backgroundColor = .clear
        IQKeyboardManager.shared.isEnabled = false
        let historyMsgs = YHAIChatDataBaseManager.shared.readHistoryUIMessages()
        messages.append(contentsOf: historyMsgs)
        
        gk_navTitle = "AI对话"
        self.view.addSubview(self.tableView)
        self.view.addSubview(self.bottomInputView)
        
        tableView.snp.makeConstraints { make in
            make.left.right.equalTo(0)
            make.top.equalTo(0)
            make.bottom.equalTo(bottomInputView.snp.top).offset(-10)
        }
        
        bottomInputView.snp.makeConstraints { make in
            make.left.right.bottom.equalToSuperview()
        }
        
        YHHUD.show(.progress(message: "加载中..."))
        self.manager.getMainChatConversationId {
            sesseionId in
            YHHUD.hide()
            self.conversationId = sesseionId
            self.getHistoryMessages(false)
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        autoResponseLocalPictureMessage()
    }
    
    func scrollToBottom() {
        self.tableView.reloadData {
            if self.messages.count > 0 {
                let indexPath = IndexPath(row: self.messages.count-1, section: 0)
                self.tableView.scrollToRow(at: indexPath, at: .bottom, animated: true)
            }
        }
    }
    
    func sendMessage(_ text: String) {
        self.manager.getMainChatConversationId {
            sesseionId in
            self.conversationId = sesseionId
            self.chatConfig.removeThinkingMessageFromChatList(&self.messages)
            let question = YHAIChatMessage.createQuestionMessage(text)
            self.messages.append(question)
            self.messages.append(self.thinkingMessage)
            self.scrollToBottom()
            self.bottomInputView.status = .loading
            self.chatConfig.disableHandleMessage = false
            self.manager.requestAI(botId: self.robotId, conversationId: self.conversationId, question:text) {
                [weak self] res, done in
                guard let self = self else { return }
                self.chatConfig.handleReceiveMessage(res, done, &messages)
                if done {
                    print("RESPONSE-DONE")
                    self.bottomInputView.status = self.bottomInputView.textView.text.isEmpty ? .disableSend : .enableSend
                }
                self.scrollToBottom()
            }
        }
    }
    
    // isPull 是否是下拉组件触发
    func getHistoryMessages(_ isPull: Bool) {
        if !isPull {
            YHHUD.show(.progress(message: "加载中..."))
        }
        viewModel.getHistoryChatMessages(botId: robotId, conversationId: conversationId, messageId: "") {
            [weak self] success, error in
            YHHUD.hide()
            guard let self = self else { return }
            self.tableView.es.stopPullToRefresh()
            
            var results = self.viewModel.historyMessages.map {
                return $0.convertToChatMessage()
            }
        
            let uuids = messages.map {
                return $0.messageId
            }
            // 过滤掉当前相同消息
            results = results.filter {
                return !uuids.contains($0.messageId)
            }
            messages.insert(contentsOf: results, at: 0)
            self.tableView.reloadData()
        }
    }
    
    func autoResponseLocalPictureMessage() {
        if isNeedAutoResponseImage {
            let msg = self.chatConfig.createRobotResponseLocalPictureMessage("ai_auto_chat_img")
            messages.append(msg)
            self.tableView.reloadData()
        }
    }
    
    func uploadEvaluationMessage(_ msg: YHAIChatMessage, callback:((Bool)->())? = nil) {
        viewModel.createMessage(conversationId: conversationId, role: "assistant", msg: msg) {
            [weak self] success, error in
            guard let self = self else { return }
            callback?(success)
        }
    }
    
    @objc func didCleanButtonClicked() {
        
        YHCommonAlertView.show("删除历史记录", "删除后记录无法恢复", "取消", "删除", fullGuestureEnable: false) {
            
        } callBack: {
            self.manager.clearGlobalConverastionId()
            self.viewModel.requestConversationId { sessionId in
                if !sessionId.isEmpty {
                    self.conversationId = sessionId
                    self.manager.saveGlobalConverastionId(sessionId)
                    self.messages.removeAll()
                    self.tableView.reloadData()
                    YHHUD.flash(message: "清除成功")
                }
            }
        }
    }
}

extension YHAIMainChatViewController: UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return messages.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        if 0 <= indexPath.row, indexPath.row < messages.count {
            let msg = messages[indexPath.row]
            let msgType = msg.getType()
            if msg.isTextMessage() {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHAITextMessageCell.cellReuseIdentifier, for: indexPath) as!  YHAITextMessageCell
                cell.message = msg
                cell.messageClick = {
                    [weak self] text in
                    guard let self = self else { return }
                    self.sendMessage(text)
                }
                return cell
                
            } else if msgType == .thinking {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHAIThinkingMessageCell.cellReuseIdentifier, for: indexPath) as!  YHAIThinkingMessageCell
                return cell
                
            } else if msgType == .productList {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHProductListMessageCell.cellReuseIdentifier, for: indexPath) as!  YHProductListMessageCell
                if let listModel = msg.body.cardsInfo {
                    cell.listModel = listModel
                }
                return cell
                
            } else if msgType == .fixProductList {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHFixProductListMessageCell.cellReuseIdentifier, for: indexPath) as!  YHFixProductListMessageCell
                if let listModel = msg.body.cardsInfo {
                    cell.listModel = listModel
                }
                return cell
                
            } else if msgType == .cardList {
                
                let cell = tableView.dequeueReusableCell(withIdentifier: YHCardMessageCell.cellReuseIdentifier, for: indexPath) as!  YHCardMessageCell
                if let listModel = msg.body.cardsInfo {
                    cell.cardListModel = listModel
                    cell.evaluationResultCallback = {
                        [weak self] dict in
                        guard let self = self else { return }
                        
                        // 生成测评结果文字 和测评结果图片 两种消息
                        var arr = [YHAIChatMessage]()
                        if let satisfy_count = dict["meet_number"] as? Int {
                            let text = "测评成功！优才新政策要求满足12项评核准则中的任意6项即可申请（即12分制满足6分），您当前满足\(satisfy_count)项，联系专属顾问咨询详细方案！"
                            let textMsg = YHAIChatMessage.createTextMesssage(text: text)
                            arr.append(textMsg)
                            self.uploadEvaluationMessage(textMsg)
                        }

                        if let imgUrl = dict["consultant_qr_code_with_text"] as? String {
                            var downUrl = ""
                            if let downloadImgUrl = dict["consultant_qr_code"] as? String {
                                downUrl = downloadImgUrl
                            }
                            let picMsg = YHAIChatMessage.createPictureMessage(imgType: 1, url: imgUrl, downloadUrl: downUrl)
                            arr.append(picMsg)
                            self.uploadEvaluationMessage(picMsg)
                        }
                        
                        if arr.count > 0 {
                            messages.append(contentsOf: arr)
                            tableView.reloadData()
                        }
                        
                    }
                }
                return cell
                
            } else if msgType == .picture {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHAIPictureMessageCell.cellReuseIdentifier, for: indexPath) as!  YHAIPictureMessageCell
                if let imgInfo = msg.body.imageInfo {
                    cell.imgInfo = imgInfo
                }
                return cell
            }
        }
        let defaultCell = tableView.dequeueReusableCell(withIdentifier: "UITableViewCell", for: indexPath)
        return defaultCell
    }

    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableView.automaticDimension
    }

    
    func tableView(_ tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat {
        return 1.0
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        let view = UIView()
        return view
    }
    
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 1.0
    }

}

extension YHAIMainChatViewController: JXSegmentedListContainerViewListDelegate {
    func listView() -> UIView {
        return view
    }
}

