//
//  YHAIRobotChatViewController.swift
//  galaxy
//
//  Created by Dufet on 2024/11/29.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//
import UIKit
import Alamofire
import IQKeyboardManagerSwift
import JXSegmentedView

class YHAIRobotChatViewController: YHBaseViewController {

    var myTitle: String = ""
    var robotId: String = ""
    var listItemId: Int = 0
    var conversationId: String = ""
    var messages:[YHAIChatMessage] = []
    var isNeedShowBannerHeader: Bool = false
    var robotType: String = ""
    let manager = YHAIRequestManager()
    let viewModel = YHAIViewModel()

    lazy var chatConfig: YHAIChatConfiguration = {
        let config = YHAIChatConfiguration()
        return config
    }()
    
    lazy var tableView: UITableView = {
        
        let tableView = UITableView(frame:.zero, style:.grouped)

        if #available(iOS 11.0, *) {
            tableView.contentInsetAdjustmentBehavior = .never
        }
        tableView.showsVerticalScrollIndicator = false
        tableView.estimatedSectionHeaderHeight = 1.0
        tableView.estimatedSectionFooterHeight = 1.0
        tableView.backgroundColor = .clear
        tableView.separatorStyle = .none
        tableView.delegate = self
        tableView.dataSource = self
        chatConfig.registerMessageGroupCells(tableView)
        return tableView
    }()
    
    lazy var bgImgView: UIImageView = {
        let v = UIImageView(image: UIImage(named: "ai_chat_bg"))
        return v
    }()
    
    lazy var bottomInputView: YHAITextInputView = {
        let v = YHAITextInputView(frame: .zero)
        v.sendBlock = {
            [weak self] text in
            guard let self = self else { return }
            self.sendMessage(text)
        }
        v.stopSendBlock = {
            [weak self] in
            guard let self = self else { return }
            self.manager.stopChat(chatId: self.manager.chatId, conversationId: self.conversationId) { success, error in
                self.bottomInputView.status = self.bottomInputView.textView.text.isEmpty ? .disableSend : .enableSend
                if success {
                    self.chatConfig.disableHandleMessage = true
                    self.chatConfig.removeThinkingMessageFromChatList(&self.messages)
                    self.bottomInputView.status = self.bottomInputView.textView.text.isEmpty ? .disableSend : .enableSend
                    self.tableView.reloadData()
                }
            }
        }
        v.keyBoardChangeBlock = {
            [weak self] isKeyBoardShow in
            guard let self = self else { return }
            self.scrollToBottom()
        }
        return v
    }()
    
    lazy var cleanBtn: UIButton = {
        let btn = UIButton()
        btn.setImage(UIImage(named: "msg_clean"), for: .normal)
        btn.YH_clickEdgeInsets = UIEdgeInsets(top: 20, left: 20, bottom: 20, right: 20)
        btn.addTarget(self, action: #selector(didCleanButtonClicked), for: .touchUpInside)
        return btn
    }()
    
    lazy var thinkingMessage: YHAIChatMessage = {
        let msg = YHAIChatMessage.createThinkingMesssage()
        return msg
    }()
    
    override func viewDidLoad() {
        
        super.viewDidLoad()
        gk_navBarAlpha = 0
        IQKeyboardManager.shared.isEnabled = false
        view.backgroundColor = .clear
        view.addSubview(bgImgView)
        
        gk_navTitle = myTitle
        gk_navTitleColor = .mainTextColor
        gk_navigationBar.addSubview(cleanBtn)
        
        self.view.addSubview(self.tableView)
        self.view.addSubview(self.bottomInputView)
        self.view.addSubview(self.bgImgView)
        self.view.insertSubview(bgImgView, belowSubview: gk_navigationBar)
        
        cleanBtn.snp.makeConstraints { make in
            make.width.height.equalTo(24)
            make.centerY.equalToSuperview()
            make.right.equalToSuperview().offset(-20)
        }
        
        bgImgView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        
        tableView.snp.makeConstraints { make in
            make.left.right.equalTo(0)
            make.top.equalTo(k_Height_NavigationtBarAndStatuBar)
            make.bottom.equalTo(bottomInputView.snp.top).offset(-10)
        }
        bottomInputView.snp.makeConstraints { make in
            make.left.right.bottom.equalToSuperview()
        }
        
        getHistoryMessages(false)
    }
    
    func scrollToBottom() {
        self.tableView.reloadData {
            if self.messages.count > 0 {
                let indexPath = IndexPath(row: self.messages.count-1, section: 0)
                self.tableView.scrollToRow(at: indexPath, at: .bottom, animated: true)
            }
        }
    }
    
    func sendMessage(_ text: String) {
        self.chatConfig.removeThinkingMessageFromChatList(&messages)
        let question = YHAIChatMessage.createQuestionMessage(text)
        messages.append(question)
        messages.append(thinkingMessage)
        self.scrollToBottom()
        self.bottomInputView.status = .loading
        self.chatConfig.disableHandleMessage = false
        self.manager.requestAI(botId: self.robotId, conversationId: self.conversationId, question:text) {
            [weak self] res, done in
            guard let self = self else { return }
            self.chatConfig.handleReceiveMessage(res, done, &messages)
            if done {
                print("RESPONSE-DONE")
                self.bottomInputView.status = self.bottomInputView.textView.text.isEmpty ? .disableSend : .enableSend
            }
            self.scrollToBottom()
        }
    }
    
    // isPull 是否是下拉组件触发
    func getHistoryMessages(_ isPull: Bool) {
        
        if !isPull {
            YHHUD.show(.progress(message: "加载中..."))
        } else {
            printLog("PULL MESSAGE HISTORY")
        }
        
        viewModel.getHistoryChatMessages(botId: robotId, conversationId: conversationId, messageId: "") {
            [weak self] success, error in
            YHHUD.hide()
            guard let self = self else { return }
            self.tableView.es.stopPullToRefresh()
            
            var results = self.viewModel.historyMessages.map {
                return $0.convertToChatMessage()
            }
    
            let uuids = messages.map {
                return $0.messageId
            }
            // 过滤掉当前相同消息
            results = results.filter {
                return !uuids.contains($0.messageId)
            }
            messages.insert(contentsOf: results, at: 0)
            self.tableView.reloadData()
            
            autoResponseLocalTextMessage()
        }
    }
    
    func autoResponseLocalTextMessage() {
        if messages.count <= 0 {
            if robotType == YHAIRobotType.sale.rawValue {
                let text = "银河集团能够为您提供全方位、一站式的香港身份规划及本地生活服务。\n在身份规划方面，我们专注于香港优才、专才、留学等身份产品服务，根据您的个人情况和需求，量身定制最合适的身份规划方案，助您轻松获取香港身份，享受香港的各项福利和优势。\n同时，我们还提供丰富的香港本地生活服务，包括教育咨询、商务支持、个人账户开设、港宝（即香港宝宝）相关服务、保险规划以及房产购置等。我们的专业团队对香港市场有着深入的了解，能够为您提供精准、高效的服务，让您在香港的生活更加便捷、舒适。\n银河集团凭借多年的行业经验和丰富的成功案例，以及专业、敬业的团队，致力于为客户提供最优质、最贴心的服务。选择银河集团，您将享受到专业、高效、便捷的服务体验，让我们携手共创美好未来。"
                let msg = self.chatConfig.createRobotResponseTextMessage(text)
                messages.append(msg)
                self.tableView.reloadData()
            }
        }
    }
    
    func uploadEvaluationMessage(_ msg: YHAIChatMessage, callback:((Bool)->())? = nil) {
        viewModel.createMessage(conversationId: conversationId, role: "assistant", msg: msg) {
            [weak self] success, error in
            guard let self = self else { return }
            callback?(success)
        }
    }
    
    @objc func didCleanButtonClicked() {
        
        YHCommonAlertView.show("删除历史记录", "删除后记录无法恢复", "取消", "删除", fullGuestureEnable: false) {
            
        } callBack: {
            self.manager.clearLobalSubRobotConversationIdForListItemId("\(self.listItemId)")
            self.viewModel.requestConversationId { sessionId in
                if !sessionId.isEmpty {
                    self.conversationId = sessionId
                    self.manager.saveLobalSubRobotConversationId(sessionId, listItemId: "\(self.listItemId)")
                    self.messages.removeAll()
                    self.tableView.reloadData()
                    YHHUD.flash(message: "清除成功")
                }
            }
        }
    }
    
    func getFlowMessages() -> [String] {
        
        if robotType == YHAIRobotType.education.rawValue {
            return  ["香港教育有哪些优势？", "去香港读书有哪些条件？", "申请香港学校费用是多少？", "了解银河教育插班服务流程", "了解银河教育插班录取率"]

        } else if robotType == YHAIRobotType.sale.rawValue {
            return  ["优才学历加分要求是什么？", "优才现在要怎么申请？", "墨尔本大学是优才合资格大学吗？", "推荐一些优才产品"]

        }
        return []
    }
    
    func getHeaderTitle() -> String {
        
        if robotType == YHAIRobotType.education.rawValue {
            return "Hello，我是香港教育宝"

        } else if robotType == YHAIRobotType.sale.rawValue {
            return  "Hello，我是新港生活规划师"
        }
        return ""
    }
    
    func getHeaderDesc() -> String {
        
        if robotType == YHAIRobotType.education.rawValue {
            return "有香港教育的问题尽管问我"

        } else if robotType == YHAIRobotType.sale.rawValue {
            return  "香港身份办理问题可以找我"
        }
        return ""
    }
    
}

extension YHAIRobotChatViewController: UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return messages.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        if 0 <= indexPath.row, indexPath.row < messages.count {
            let msg = messages[indexPath.row]
            let msgType = msg.getType()
            if msg.isTextMessage() {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHAITextMessageCell.cellReuseIdentifier, for: indexPath) as!  YHAITextMessageCell
                cell.message = msg
                cell.messageClick = {
                    [weak self] text in
                    guard let self = self else { return }
                    self.sendMessage(text)
                }
                return cell
                
            } else if msgType == .thinking {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHAIThinkingMessageCell.cellReuseIdentifier, for: indexPath) as!  YHAIThinkingMessageCell
                return cell
                
            } else if msgType == .productList {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHProductListMessageCell.cellReuseIdentifier, for: indexPath) as!  YHProductListMessageCell
                if let listModel = msg.body.cardsInfo {
                    cell.listModel = listModel
                }
                return cell
                
            } else if msgType == .fixProductList {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHFixProductListMessageCell.cellReuseIdentifier, for: indexPath) as!  YHFixProductListMessageCell
                if let listModel = msg.body.cardsInfo {
                    cell.listModel = listModel
                }
                return cell
                
            } else if msgType == .cardList {
                
                let cell = tableView.dequeueReusableCell(withIdentifier: YHCardMessageCell.cellReuseIdentifier, for: indexPath) as!  YHCardMessageCell
                if let listModel = msg.body.cardsInfo {
                    cell.cardListModel = listModel
                    cell.evaluationResultCallback = {
                        [weak self] dict in
                        guard let self = self else { return }
                        
                        // 生成测评结果文字 和测评结果图片 两种消息
                        var arr = [YHAIChatMessage]()
                        if let satisfy_count = dict["meet_number"] as? Int {
                            let text = "测评成功！优才新政策要求满足12项评核准则中的任意6项即可申请（即12分制满足6分），您当前满足\(satisfy_count)项，联系专属顾问咨询详细方案！"
                            let textMsg = YHAIChatMessage.createTextMesssage(text: text)
                            arr.append(textMsg)
                            self.uploadEvaluationMessage(textMsg)
                        }

                        if let imgUrl = dict["consultant_qr_code_with_text"] as? String {
                            var downUrl = ""
                            if let downloadImgUrl = dict["consultant_qr_code"] as? String {
                                downUrl = downloadImgUrl
                            }
                            let picMsg = YHAIChatMessage.createPictureMessage(imgType: 1, url: imgUrl, downloadUrl: downUrl)
                            arr.append(picMsg)
                            self.uploadEvaluationMessage(picMsg)
                        }
                        
                        if arr.count > 0 {
                            messages.append(contentsOf: arr)
                            tableView.reloadData()
                        }
                        
                    }
                }
                return cell
                
            } else if msgType == .picture {
                let cell = tableView.dequeueReusableCell(withIdentifier: YHAIPictureMessageCell.cellReuseIdentifier, for: indexPath) as!  YHAIPictureMessageCell
                if let imgInfo = msg.body.imageInfo {
                    cell.imgInfo = imgInfo
                }
                return cell
            }
        }
        let defaultCell = tableView.dequeueReusableCell(withIdentifier: "UITableViewCell", for: indexPath)
        return defaultCell
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableView.automaticDimension
    }

    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        
        if !isNeedShowBannerHeader {
            return UIView()
        }
        
        let view = YHAIChatBannerView()
        view.titleLabel.text = getHeaderTitle()
        view.descLabel.text = getHeaderDesc()
        view.bannerArr = self.getBannerForRobotType(robotType)
        view.messages = getFlowMessages()
        view.selectFlowMsgBlock = {
            [weak self] text in
            guard let self = self else { return }
            self.sendMessage(text)
        }
        view.selectBannerItemBlock = {
            [weak self] model in
            guard let self = self else { return }
            var text = ""
            if !model.msg.isEmpty {
                text = model.msg
                
            } else if !model.desc.isEmpty {
                text = model.desc
            }

            self.sendMessage(text)
        }
        return view
    }
    
    func getBannerForRobotType(_ robotType: String) -> [YHAIChatBannerItem] {
        
        if robotType == YHAIRobotType.sale.rawValue {
            return  [YHAIChatBannerItem(id: 0, title: "了解银河集团", desc: "香港身份生活一站式服务平台", msg: "银河集团，能够为我提供什么？"),
                     YHAIChatBannerItem(id: 1, title: "香港身份智能评估", desc: "60s快速评估，了解自身条件是否符合", msg: "开始身份办理评估"),
                     YHAIChatBannerItem(id: 2, title: "银河产品矩阵", desc: "香港身份、生活多样产品", msg: "介绍一下银河的产品"),]
           
        }
        if robotType == YHAIRobotType.education.rawValue {
            return  [YHAIChatBannerItem(id: 0, title: "幼中小学升学", desc: "去香港插班需要考核哪些"),
                     YHAIChatBannerItem(id: 1, title: "大学升学", desc: "DSE分数和Alevel的换算关系"),
                     YHAIChatBannerItem(id: 2, title: "银河教育服务", desc: "银河教育插班成功率如何？"),]
        }
        return []
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        let view = UIView()
        return view
    }
    
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        if !isNeedShowBannerHeader {
            return 1.0
        }
        return 360.0
    }
}

extension YHAIRobotChatViewController: JXSegmentedListContainerViewListDelegate {
    func listView() -> UIView {
        return view
    }
}


