//
//  YHAICardItemView.swift
//  galaxy
//
//  Created by Dufet on 2024/12/3.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit

class YHAICardItemView: UIView {
    
    var cardModel = YHAIListItemModel() {
        didSet {
            cardImgView.sd_setImage(with: URL(string: cardModel.cover), placeholderImage: UIImage(named: "global_default_image"))
            cardTitleLabel.text = cardModel.title
            describeLabel.text = cardModel.description
        }
    }
    
    lazy var lineView: UIView = {
        let v = UIView()
        v.backgroundColor = .init(hex: 0xE9ECF0)
        return v
    }()
    
    lazy var cardImgView: UIImageView = {
        let v = UIImageView(image: UIImage(named: "global_default_image"))
        return v
    }()
    
    lazy var cardTitleLabel:UILabel = {
        let lable = UILabel()
        lable.textColor = UIColor.mainTextColor
        lable.textAlignment = .left
        lable.font = UIFont.PFSC_B(ofSize:16)
        lable.numberOfLines = 0
        lable.text = "卡片标题"
        return lable
    }()
    
    lazy var describeLabel:UILabel = {
        let lable = UILabel()
        lable.textColor = UIColor.init(hex: 0x8993A2)
        lable.textAlignment = .left
        lable.font = UIFont.PFSC_R(ofSize:12)
        lable.numberOfLines = 0
        lable.text = "卡片描述文字"
        return lable
    }()
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        createUI()
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func createUI() {
        
        let tap = UITapGestureRecognizer(target: self, action: #selector(didItemViewClicked))
        self.addGestureRecognizer(tap)
        
        self.addSubview(lineView)
        self.addSubview(cardImgView)
        self.addSubview(cardTitleLabel)
        self.addSubview(describeLabel)
        
        lineView.snp.makeConstraints { make in
            make.left.equalTo(16)
            make.right.equalTo(-16)
            make.top.equalTo(0)
            make.height.equalTo(0.5)
        }
        
        cardImgView.snp.makeConstraints { make in
            make.left.equalTo(16)
            make.right.equalTo(-16)
            make.top.equalTo(lineView.snp.bottom).offset(16)
            make.height.equalTo(303)
        }
        
        cardTitleLabel.snp.makeConstraints { make in
            make.top.equalTo(cardImgView.snp.bottom).offset(10)
            make.left.equalTo(16)
            make.right.equalTo(-16)
        }
        
        describeLabel.snp.makeConstraints { make in
            make.top.equalTo(cardTitleLabel.snp.bottom).offset(4)
            make.left.equalTo(16)
            make.right.equalTo(-16)
            make.bottom.equalTo(-16)
        }
    }
    
    @objc func didItemViewClicked() {
        
        YHAIJumpPageTool.jumpPageWithType(0, mode: cardModel.redirectMode, path: cardModel.redirectPath) {
            dict in
        }
    }
}
